%
% Convergence experiment with synthetic data
% ------------------------------------------
%
% In this script we reproduce the convergence results for a noisy spiral data
% in the file "demo_toy_data.mat" (see show_syntetic_data.m or generate_spiral.m)
%
% We initialize SPCA with sensible parameters for this manifold. These parameters 
% were obtained by minimizing the projection error, see the procedure described in 
% the Technical Report http://isp.uv.es/docs/.
%
%     Initialization takes 3 secs. in a processor of a Dell PowerEdge 2900
%
% Then, we transform 500 test samples and monitorize their convergence by 
% inspecting the evolution of the approximation error using 
% "spca_2_convergencia.m" that returns the approximation error for each
% sample and iteration.
%
%     Transform takes about 1.5 secs./sample in a processor of a Dell PowerEdge 2900
%
% This program generates the file "convergence_synthetic.mat" in the destination
% folder provided by the user.
%
% These data (together with data coming from exp_converg_texture.m and 
% exp_converg_color.m) have to be analyzed by "convergence_experiments.m" to
% obtain the convergence plots shown in the paper.
%

clear all

destination_folder = '/media/disk/vista/Papers/aranyas/general_y_analiticas/general/TNN_2012_reproducible/exp_assumptions/convergence/';

% DATA (noisy spiral with increasing noise along the way)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load demo_toy_data

dat = dat(:,1:2000);
dat_test = dat_test(:,1:500);


N_data = length(dat(1,:));          % Number of training samples
N_samples = length(dat_test(1,:));  % Number of test samples
  
    %%%%%%%%%%%%%%%%%%%
    % PARAMETERS OF SPCA
    %%%%%%%%%%%%%%%%%%%
    
    criterio = 1;   % metric criterion (1 stands for euclidean metric, 2 stands for infomax, and 3 stands for error minimization)
    tol = 0.001*(0.05/0.0018);             % tolerance in for the inversion error
    tau = 0.5;              % step size in each PC
    Nv = 0.1*N_data;        % k-neighbors
    pto = [-2;-8.5];        % coordinate origin
    q  = 0.05;              % stiffness
    Btotal = 7;             % number of bits to be distributed among dimensions.

    % If zero is given to the following variables, the initialization
    % function looks for a default value for them.
    
    Nbins = 0;    % Number of bins (resolution) in each dimension e.g. Nbins = [N1, N2, ..., Nd]
    DIST = 0;     % Euclidean distance threshold for the "out-of-the-manifold" condition
    Aref = 0;     % Reference matrix with the local PCA directions at the origin

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   INITIALIZATION (on training data):
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    [init_param] = initialize_SPCA_2(dat,pto,Nv,tau,q,criterio,Btotal,Nbins,DIST,Aref);

    figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
    plotea_eje(init_param.eucl_PC,'k-')
    title(['Principal Curve euclidean metric'])

    figure,plot(dat(1,:),dat(2,:),'.'), axis square, axis equal
    plotea_eje(init_param.PC,'k-')
    title(['Principal Curve modified metric'])

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   TRANSFORM OF TEST DATA:
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % R2          = PCU transform of test data

    TT = 0;
    for nn = 1:N_samples
        tic
           
        [pto_proximo21(:,nn) R21(:,nn) pto_proximo11(:,nn) R11(:,nn) EEE(nn).errores] = spca_2_convergencia(dat,dat_test(:,nn),init_param,tol);

        aux = toc;
        
        TT = TT +aux;
        ['sample: ' num2str(nn) '  sample-time: ' num2str(aux) '  estimated remaining time: '  num2str((N_samples-nn)*TT/nn)]
        
        save([destination_folder,'convergence_synthetic.mat']) 
    end
    
