%
% Convergence experiment with natural texture data
% -----------------------------------------------
%
% In this script we reproduce the convergence results for texture data.
% We use the texture data in the file "same_orientation_data.mat" 
% (see texture_masking_data.m)
%
% We initialize SPCA with sensible parameters for image manifolds (which
% are not curved). These parameters could also be obtained by minimizing the 
% projection error,see the procedure described in 
% the Technical Report http://isp.uv.es/docs/.
%
%     Initialization takes 25 secs. in a processor of a Dell PowerEdge 2900
%
% Then, we transform 1000 test samples and monitorize their convergence by 
% inspecting the evolution of the approximation error using 
% "spca_2_convergencia.m" that returns the approximation error for each
% sample and iteration.
%
%     Transform takes about 9 secs./sample in a processor of a Dell PowerEdge 2900
%
% This program generates the file "convergence_texture.mat" in the destination
% folder provided by the user.
%
% These data (together with data coming from exp_converg_synthetic.m and 
% exp_converg_color.m) have to be analyzed by "convergence_experiments.m" to
% obtain the convergence plots shown in the paper.
%


clear all

destination_folder = '/media/disk/vista/Papers/aranyas/general_y_analiticas/general/TNN_2012_reproducible/exp_assumptions/convergence/';

% DATA (image data living close to the 3d subspace defined by the 
% DC component and two low frequency diagonal directions)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load 'same_orientation_data' 'dat_fd_proxim'

ind = randperm(size(dat_fd_proxim,2));

dat = dat_fd_proxim(:,ind(1:2000));
dat_test = dat_fd_proxim(:,ind(2001:3000));


N_data = length(dat(1,:));          % Number of training samples
N_samples = length(dat_test(1,:));  % Number of test samples

  
    %%%%%%%%%%%%%%%%%%%
    % PARAMETERS OF SPCA
    %%%%%%%%%%%%%%%%%%%
    
    criterio = 2;   % metric criterion (1 stands for euclidean metric, 2 stands for infomax, and 3 stands for error minimization)
    tol = 0.001*1.13;          % tolerance in for the inversion error
    tau = 0.25;              % step size in each PC
    Nv = [round(0.2*N_data)];        % k-neighbors
    pto = [0 0 0]';        % coordinate origin
    q  = 0.06;              % stiffness
    Btotal = 9;             % number of bits to be distributed among dimensions.

    % If zero is given to the following variables, the initialization
    % function looks for a default value for them.
    
    Nbins = 0;    % Number of bins (resolution) in each dimension e.g. Nbins = [N1, N2, ..., Nd]
    DIST = 0;     % Euclidean distance threshold for the "out-of-the-manifold" condition
    Aref = eye(3);     % Reference matrix with the local PCA directions at the origin

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   INITIALIZATION (on training data):
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    [init_param] = initialize_SPCA_2(dat,pto,Nv,tau,q,criterio,Btotal,Nbins,DIST,Aref);

    figure,plot3(dat(1,:),dat(2,:),dat(3,:),'.'), axis square, axis equal
    plotea_eje(init_param.eucl_PC,'k-')
    title(['Principal Curve euclidean metric'])

    figure,plot3(dat(1,:),dat(2,:),dat(3,:),'.'), axis square, axis equal
    plotea_eje(init_param.PC,'k-')
    title(['Principal Curve modified metric'])

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%
    %%   TRANSFORM OF TEST DATA:
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % R2          = PCU transform of test data

    TT = 0;
    for nn = 1:N_samples
        tic
           
        [pto_proximo21(:,nn) R21(:,nn) pto_proximo11(:,nn) R11(:,nn) EEE(nn).errores] = spca_2_convergencia(dat,dat_test(:,nn),init_param,tol);

        aux = toc;
        
        TT = TT +aux;
        ['sample: ' num2str(nn) '  sample-time: ' num2str(aux) '  estimated remaining time: '  num2str((N_samples-nn)*TT/nn)]
        save([destination_folder,'convergence_texture.mat']) 
    end

    figure,plot(R11(1,:),R11(2,:),'.'), axis square, axis equal
    title(['Transformed data without modifying the local metric'])

    figure,plot(R21(1,:),R21(2,:),'.'), axis square, axis equal
    title(['Transformed data with metric-criterion ' num2str(criterio)])
